<?php
//static function library
class SMClientLibrary
{
	/********************
	*	Date Functions	*
	********************/
	/**
	*	Takes an email formatted date, converts it to the given timezone, and formats it
	*
	*	@param (string) $date The raw date from the email
	*	@param (string=gmt) $tz The GMT offset of the timezone to convert the date to.
	*	@param (string) $format The format to use. See date()
	*	@param (boolean) $relative Whether or not to use relative time (ie today, yesterday) for recent email
	*
	*	@return (mixed) The formatted date, unix timestamp if format = '', or an array of minutes, hours, and days
	*		if relative is true
	*/
	static function formatDate($date, $tz = '+0000', $format = 'F j, Y, g:i a', $relative = false)
	{
		$ts = strtotime($date);
		$tz_offset = tzToSeconds($tz);
		
		$time = $ts + $tz_offset;
		
		if($relative)
		{
			$diff = time() - $time;
			$relative = array();
			$relative['days']		= (int) (  $diff / 60 / 60 / 24);
			$relative['hours']		= (int) ( ($diff / 60 / 60) - ($relative['days'] * 24) );
			$relative['minutes']	= (int) ( ($diff / 60) - ($relative['hours'] * 60) - ($relative['days'] * 24 * 60) );
			
			return $relative;
		}
		
		if($format == '')
			return $time;
			
		return date($format, $time);
	}
	
	/*
	*	Takes a timestamp formatted as +/-#### ie -0800 and converts it to seconds from GMT
	*/
	private static function tzToSeconds($tz)
	{
		$sign = ($tz[0] == '+') ? 1 : -1;
		$hours = (int) ($tz[1] . $tz[2]);
		$minutes = (int) ($tz[3] . $tz[4]);
		
		$seconds = $sign * ($hours * 60 * 60 + $minutes * 60);
		return $seconds;
	}
}
?>