<?php
class SimpleMailMessage
{
	public $id;
	private $options;
	private $mbox;
	

	public $structure;
	private $header;
	private $content;
	
	public $rawHeader;
	public $from;
	public $to;
	public $cc;
	public $date;
	public $subject;
	
	public $plaintext;
	public $html;
	
	public $attachments;
	public $embed;
	
	public function __construct($mbox, $id, $uid = true, $peek = false)
	{
		$this->mbox = $mbox;
		$this->id = $id;
		$this->uid = $uid;
		
		$this->attachments = array();
		
		$options = ($uid) ? FT_UID : 0;
		
		$this->rawHeader = imap_fetchheader($this->mbox, $id, $options);
		if(!$this->rawHeader)
			throw new Exception('Could not retrieve message header');
			
		$this->header = imap_rfc822_parse_headers($this->rawHeader);
		
		//print_r($this->header);
		$this->from 	= isset($this->header->fromaddress) 	? @iconv_mime_decode($this->header->fromaddress) 	: '';
		$this->to 		= isset($this->header->toaddress) 		? @iconv_mime_decode($this->header->toaddress) 		: '';
		$this->date 	= isset($this->header->date) 			? @iconv_mime_decode($this->header->date)			: '';
		$this->subject 	= isset($this->header->subject) 		? @iconv_mime_decode($this->header->subject) 		: '';
		$this->embed = array();
		
		
		$options = ($uid) ? FT_UID : 0;
		$options |= ($peek) ? FT_PEEK : 0;
		
		$this->options = $options;
		
		$this->structure = imap_fetchstructure($this->mbox, $id, $options);
		if(!$this->structure)
			throw new Exception('Could not retrieve message structure');
			
		if(!$this->structure->parts)
		{
			$this->content = imap_fetchbody($this->mbox, $this->id, 1, $this->options);
			if(strtolower($this->structure->subtype) == 'html')
				$this->html = $this->formatHtml($this->content);
			else
				$this->plaintext = $this->formatPlainText($this->content);
		}
		else
			$this->parseStructure($this->structure->parts);
			
		if(!$this->plaintext)
			$this->plaintext = $this->formatPlainText(@base64_decode($this->html));
			
		if(!$this->html)
			$this->html = $this->formatHtml($this->content);
	}
	
	private function parseStructure($parts, $parent = 0)
	{
		//print_r($this->structure); //!!
		
		$i = 1;
		foreach($parts as $part)
		{
			$partno = ($parent) ? "$parent.$i" : $i;
			$params = array();
			
			if ($part->parameters)
				foreach ($part->parameters as $p)
					$params[ strtolower( $p->attribute ) ] = $p->value;
			if ($part->dparameters)
				foreach ($part->dparameters as $p)
					$params[ strtolower( $p->attribute ) ] = $p->value;
			
			if($part->parts)
				$this->parseStructure($part->parts, $partno);
			elseif($part->disposition == 'attachment' || $params['filename'] || $params['name'])
			{
				$filename = ($params['filename'])? $params['filename'] : $params['name'];
				$this->attachments[] = (object) array(	'filename' 	=> $filename, 
														'bytes' 	=> $part->bytes, 
														'type' 		=> $part->subtype, 
														'id' 		=> urlencode($partno),
														'encoding'	=> $part->encoding	);
				//print_r($part);
			}
			elseif($part->type == 0)
			{
				$this->content = imap_fetchbody($this->mbox, $this->id, $partno, $this->options);
				if(strtolower($part->subtype) == 'html')
					$this->html = $this->formatHtml($this->content, $part->encoding);
				else
					$this->plaintext = $this->formatPlainText($this->content, $part->encoding);
			}
			elseif($part->type == 2)
			{
				$this->embed[] = strip_tags(imap_fetchbody($this->mbox, $this->id, $partno, $this->options));
			}

			$i++;
		}
	}
	
	public function formatPlainText($content, $encoding = 4)
	{
		if($encoding == 3)
			$content = base64_decode(trim($content));
		elseif($encoding == 4)
			$content = quoted_printable_decode(trim($content));

		$content = strip_tags($content);
		$content = htmlentities(str_replace("\r", '', $content));
		$content = nl2br(trim($content));
		$content = preg_replace("/(<br\s*\/?>\s*){2}+/", '<br />', $content);

		return $this->linkify($content);
	}
	
	public function formatHtml($content, $encoding = 4)
	{
		if($encoding == 3)
			return $content;
		return base64_encode(quoted_printable_decode($content));
	}
	
	private function decode($data, $encoding = 4)
	{
		if($encoding == 4)
			return quoted_printable_decode(trim($data));
		elseif($encoding == 3)
			return base64_decode(trim($data));
		
		return $data;
	}
	
	private function linkify($content)
	{
		if(preg_match_all("/(^|[\n ])([\w]*?)(http(s)?:\/\/[\w]+[^ \,\"\n\r\t<]*)/is", $content, $m))
		{
			$matches = array();
			foreach($m[0] as $match)
			{
				if(trim($match))
					$matches[] = rtrim(trim($match), '.,);:]}?!>');
			}
			
			$matches = array_unique($matches);
			usort($matches, create_function('$a,$b', 'return strlen($b)-strlen($a);'));
			
			for($i = 1; $i <= count($matches); $i++)
			{
				$replace = ':::linkify_match_' . $i . ':::';
				$content = str_replace($matches[$i-1], '<a href="' . $replace . '" target="_blank">' . $replace . '</a>', $content);
			}
				
			for($i = 1; $i <= count($matches); $i++)
			{
				$find = ':::linkify_match_' . $i . ':::';
				$content = str_replace($find, $matches[$i-1], $content);
			}
		}
		
		if(preg_match_all("/(^|[\n ])([a-z0-9&\-_\.]+?)@([\w\-]+\.([\w\-\.]+)+)/i", $content, $m))
		{
			$matches = array();
			foreach($m[0] as $match)
			{
				if(trim($match))
					$matches[] = rtrim(trim($match), '.,);:]}?!>');
				//$content = str_replace($match, '<a class="address-link" href="mailto:' . $match . '">' . $match . '</a>', $content);
			}
			
			$matches = array_unique($matches);
			usort($matches, create_function('$a,$b', 'return strlen($b)-strlen($a);'));
			
			for($i = 1; $i <= count($matches); $i++)
			{
				$replace = ':::linkify_match_' . $i . ':::';
				$content = str_replace($matches[$i-1], '<a href="mailto:' . $replace . '" class="address-link">' . $replace . '</a>', $content);
			}
				
			for($i = 1; $i <= count($matches); $i++)
			{
				$find = ':::linkify_match_' . $i . ':::';
				$content = str_replace($find, $matches[$i-1], $content);
			}
		}
		//$content = preg_replace("/(^|[\n ])([\w]*?)(http(s)?:\/\/[\w]+[^ \,\"\n\r\t<]*)/is", "$1$2<a href=\"$3\" >$3</a>", $content);
		//$content = preg_replace("", "$1<a href=\"mailto:$2@$3\">$2@$3</a>", $content);
		return $content;
	}
}

?>