<?php
class SMLibrary
{
	/****************
	*	Settings	*
	****************/
	
	public static function default_settings()
	{
		$defaults = array(
			'multi_user'		=> false,
			'host'				=> '',
			'force_login'		=> false,
			'username'			=> '',
			'username_suffix'	=> '',
			'password'			=> '',
			'flags' 			=> array(	'imap' 				=> 1,
											'pop3' 				=> 0,
											'anonymous' 		=> 0,
											'debug' 			=> 0,
											'secure' 			=> 0,
											'nntp' 				=> 0,
											'norsh'				=> 0,
											'ssl' 				=> 0,
											'validate-cert' 	=> 0,
											'novalidate-cert' 	=> 1,
											'tls' 				=> 0,
											'notls' 			=> 0,
											'readonly' 			=> 0 ),
			
			'notifications'		=> true,
			'admin_bar'			=> true,
			'messages_per_page'	=> 20,
			'signature'			=> '',
			'menu_position'		=> null,
			
			'version'			=> SimpleMailPlugin::VERSION,
			'ready'				=> 0,
			'boxes'				=> new stdClass(),
			'build'				=> 0
		);
		
		return (object) $defaults;
		/*
		$selected = array_combine($keys, array_fill(0, count($keys), null));
		return (object) array_intersect_key($defaults, $selected);
		*/
	}
	
	public static function default_user_settings()
	{
		static $defaults = array(
			'build'				=> 0
		);
		
		return (object) $defaults;
	}
	
	public static function all_flags()
	{
		return array(	'imap' 				=> 0,
						'pop3' 				=> 0,
						'anonymous' 		=> 0,
						'debug' 			=> 0,
						'secure' 			=> 0,
						'nntp' 				=> 0,
						'norsh'				=> 0,
						'ssl' 				=> 0,
						'validate-cert' 	=> 0,
						'novalidate-cert' 	=> 0,
						'tls' 				=> 0,
						'notls' 			=> 0,
						'readonly' 			=> 0 );
	}
	
	public static function process_settings(&$settings)
	{
		//flags
		if(isset($settings->flags))
		{
			$flags = SMLibrary::all_flags();
			if(is_array($settings->flags))
			{
				foreach($settings->flags as $flag)
					$flags[$flag] = 1;
			}
			$settings->flags = $flags;
		}
		
		//menu position
		if(isset($settings->menu_position))
		{
			if($settings->menu_position === '')
				$settings->menu_position = null;
			else
				$settings->menu_position = (int) $settings->menu_position;
		}
		
		//messages per page
		if(isset($settings->messages_per_page))
		{
			if($settings->messages_per_page < 1)
				$settings->messages_per_page = 20; //set to default
		}
		
		//signature
		if(isset($settings->signature))
		{
			$settings->signature = html_entity_decode($settings->signature);
		}
		
		//force login
		if(isset($settings->force_login) && $settings->force_login)
		{
			if(isset($settings->username))
				unset($settings->username);
			if(isset($settings->password))
				unset($settings->password);
		}
	}
	
	public static function isSettings()
	{
		global $sm_plugin;
		if(isset($_GET['page']) && $_GET['page'] == 'simplemail_settings')
			return true;
		
		if($sm_plugin->settings->multi_user && SMLibrary::isProfile())
			return true;
			
		return false;
	}
	
	public static function isProfile()
	{
		global $pagenow;
		
		if($pagenow == 'profile.php' || $pagenow == 'user-edit.php')
			return true;
			
		return false;
	}
	
	public static function prep_user_settings($user_settings = null)
	{
		global $sm_plugin;
		if(!$user_settings)
		{
			global $current_user;
			get_currentuserinfo();
			
			$user_settings = get_user_meta($current_user->ID, 'simplemail_settings', true);
			//print_r($user_settings);exit;
			if($user_settings && $user_settings->build != $sm_plugin->settings->build)
				SMLibrary::syncUserSettings($user_settings, $current_user->ID);
		}
		
		if(!$user_settings)
		{
			$user_settings = get_option('simplemail_user_settings');
		}
		
		foreach($user_settings as $property => $value)
		{
			$sm_plugin->settings->$property = $value;
		}
	}
	
	public static function syncUserSettings(&$user_settings, $id)
	{
		global $sm_plugin;
		$current_fields = get_option('simplemail_user_settings');
		//$ready = $user_settings->ready;
		$ignore = array('ready', 'boxes');
		
		foreach($user_settings as $field => $val)
		{
			if(!in_array($field, $ignore) && !isset($current_fields->$field))
				unset($user_settings->$field);
		}
		
		foreach($current_fields as $field => $val)
		{
			if(!isset($user_settings->$field))
				$user_settings->$field = $current_fields->$field;
		}
		
		$user_settings->build = $sm_plugin->settings->build;
		
		update_user_meta($id, 'simplemail_settings', $user_settings);
	}
	
	public static function determine_if_ready()
	{
		global $sm_plugin;
		if($sm_plugin->settings->host && $sm_plugin->settings->username && $sm_plugin->settings->password)
		{
			$sm_plugin->startClient();
			
			if($sm_plugin->client->quickConnect())
			{
				$sm_plugin->settings->ready = true;
				
				$boxes = $sm_plugin->client->getMailBoxes(true);
				$sm_plugin->settings->boxes = new stdClass();
				
				foreach($boxes as $box)
				{
					if(strtolower($box->raw) == 'inbox')
						$sm_plugin->settings->boxes->inbox = $box->raw;
					elseif(stripos($box->raw, 'sent') !== false)
						$sm_plugin->settings->boxes->sent = $box->raw;
					elseif(stripos($box->raw, 'draft') !== false)
						$sm_plugin->settings->boxes->drafts = $box->raw;
					elseif(stripos($box->raw, 'trash') !== false)
						$sm_plugin->settings->boxes->trash = $box->raw;
				}
				
				//print_r($sm_plugin->settings);exit;
				
				if(!$sm_plugin->settings->boxes->inbox)
					SMLibrary::nag('<strong>' . __('Could not determine which mailbox to use as the Inbox. Please specify the inbox under the Mailbox settings.') . '</strong>');
			}
			else
			{
				SMLibrary::nag('<strong>' . __('Could not connect to the mail server. Please check your settings') . '</strong>');
			}
		}
		elseif(!$sm_plugin->settings->force_login)
			SMLibrary::nag('<strong>' . __('You have not supplied all of the settings necessary to connect to the mail server.') . '</strong>');

		return $sm_plugin->settings->ready;
	}
	
	public static function assignBoxTypes($boxes)
	{
	
	}
	
	/****************
	*	Mailbox		*
	****************/
	
	static function isMailbox()
	{
		if(isset($_GET['page']))
		{
			if(strpos($_GET['page'], 'simplemail') === 0 && 
				(	!in_array( $_GET['page'], array('simplemail_settings', 'simplemail_addresses', 'simplemail_compose') ) ||
					$_GET['page'] == 'simplemail_compose' && isset($_GET['box'])	)
			)
			{
				return true;
			}
		}
		
		
		return false;
	}
	
	static function getMailbox()
	{
		global $sm_plugin;
		
		if(!isset($_GET['page']))
			return false;
			
		if(isset($_GET['box']))
			$key = 'box';
		else
			$key = 'page';
			
		if($_GET[$key] == 'simplemail')
			return $sm_plugin->client->inbox;
			
		$mailbox = str_replace('simplemail_', '', $_GET[$key]);
		
		//return $mailbox;
		
		return $sm_plugin->client->slugToMailbox($mailbox);
	}
	
	/****************
	*	Misc		*
	****************/
	
	public static function processLogin()
	{
		global $sm_plugin;
		if(isset($_POST['sm_login_submit']))
		{
			$sm_plugin->settings->username = $_POST['sm_username'];
			$sm_plugin->settings->password = $_POST['sm_password'];
			
			if(SMLibrary::determine_if_ready())
			{
				//SMLibrary::nag('<strong>' . __('') . '</strong>');
				$_SESSION['sm_username'] = $_POST['sm_username'];
				$_SESSION['sm_password'] = $_POST['sm_password'];
				$sm_plugin->settings->ready = true;
			}
		}
		elseif(isset($_SESSION['sm_username']))
		{
			$sm_plugin->settings->username = $_SESSION['sm_username'];
			$sm_plugin->settings->password = $_SESSION['sm_password'];
			$sm_plugin->settings->ready = true;
		}
	}
	
	public static function nag($content)
	{
		add_action('admin_notices', create_function('', "echo '<div class=\"updated\"><p>$content</p></div>';"));
	}	
	
	public static function format_email_notice($count)
	{
		return "<span class='unread-email count-$count'><span class='email-count'>" . number_format_i18n($count) . '</span></span>';
	}
	
		public static function get_roles()
	{
		global $wp_roles;
       	$roles = $wp_roles->get_names();
		
		return $roles;
	}
	
	public static function can_access_email()
	{
		global $wp_roles;
		$can = array();
		foreach(self::get_roles() as $role => $display)
		{
			$role = get_role($role);

			if(isset($role->capabilities['access_email']) && $role->capabilities['access_email'])
				$can[] = $role->name;
		}
		
		return $can;
	}
	
	public static function content_type($filename)
	{
		static $types = array(

		'txt' => 'text/plain',
		'htm' => 'text/html',
		'html' => 'text/html',
		'php' => 'text/html',
		'css' => 'text/css',
		'js' => 'application/javascript',
		'json' => 'application/json',
		'xml' => 'application/xml',
		'swf' => 'application/x-shockwave-flash',
		'flv' => 'video/x-flv',

		// images
		'png' => 'image/png',
		'jpe' => 'image/jpeg',
		'jpeg' => 'image/jpeg',
		'jpg' => 'image/jpeg',
		'gif' => 'image/gif',
		'bmp' => 'image/bmp',
		'ico' => 'image/vnd.microsoft.icon',
		'tiff' => 'image/tiff',
		'tif' => 'image/tiff',
		'svg' => 'image/svg+xml',
		'svgz' => 'image/svg+xml',

		// archives
		'zip' => 'application/zip',
		'rar' => 'application/x-rar-compressed',
		'exe' => 'application/x-msdownload',
		'msi' => 'application/x-msdownload',
		'cab' => 'application/vnd.ms-cab-compressed',

		// audio/video
		'mp3' => 'audio/mpeg',
		'qt' => 'video/quicktime',
		'mov' => 'video/quicktime',

		// adobe
		'pdf' => 'application/pdf',
		'psd' => 'image/vnd.adobe.photoshop',
		'ai' => 'application/postscript',
		'eps' => 'application/postscript',
		'ps' => 'application/postscript',

		// ms office
		'doc' => 'application/msword',
		'rtf' => 'application/rtf',
		'xls' => 'application/vnd.ms-excel',
		'ppt' => 'application/vnd.ms-powerpoint',

		// open office
		'odt' => 'application/vnd.oasis.opendocument.text',
		'ods' => 'application/vnd.oasis.opendocument.spreadsheet',
		);
		 
		$ext = substr(strrchr($filename, '.'), 1);
		if(isset($types[$ext]))
			return $types[$ext];
			
		return 'application/octet-stream';
	}
}
?>