<?php
define('SM_DEBUG', false);

require_once('library.php');
if(function_exists('imap_open')) :
require_once('ui/menus.php');
require_once('client/client.php');
require_once('admin_bar.php');
require_once('ui/ajax.php');

class SimpleMailPlugin
{
	const VERSION = '1.0.4';
	const HOME = 'http://tinsology.net/plugins/wp-simplemail/';
	const FORUM = 'http://forums.tinsology.net/';
	public $settings;
	public $pages;
	
	public $client;
	
	public $url;
	public $dir;
	
	public $current_mailbox;
	
	public function __construct()
	{
		add_action('init', array($this, 'init'), 2);
		add_action('init', array($this, 'connect'), 3);
		
		register_activation_hook(__FILE__, array($this, 'install'));
		register_deactivation_hook(__FILE__, array($this, 'uninstall'));
		
		$this->url = plugins_url('', __FILE__);
		$this->dir = dirname(__FILE__);
		
		$this->pages = (object) array(	'compose'	=> admin_url('admin.php?page=simplemail_compose'),
										'settings'	=> admin_url('admin.php?page=simplemail_settings'),
										'inbox'		=> admin_url('admin.php?page=simplemail')	);
	}
	
	public function init()
	{
		if(is_user_logged_in() && (current_user_can('access_email') || current_user_can('manage_options')))
		{
			$this->settings = get_option('simplemail_settings');
			if(!$this->settings)
			{
				$this->install();
				$this->settings = get_option('simplemail_settings');
			}
			elseif(!$this->settings->flags)
			{
				$defaults = SMLibrary::default_settings();
				$this->settings->flags = $defaults->flags;
				update_option('simplemail_settings', $this->settings);
			}
			
			if($this->settings->force_login)
			{
				if(!session_id())
					session_start();
					
				SMLibrary::processLogin();
			}
			
			if($this->ready())
				$this->startClient();
				
			if(isset($this->settings->legacy) && $this->settings->legacy && is_admin())
			{
				SMLibrary::nag('WP SimpleMail has just been updated. Some of the changes made since the last version require that your settings be reset. 
								Go to the <a href="' . $this->pages->settings . '">Settings Page</a> to configure SimpleMail Version ' . SimpleMailPlugin::VERSION);
				$this->settings->legacy = 0;
				update_option('simplemail_settings', $this->settings);
			}
			elseif(isset($this->settings->update_nag) && $this->settings->update_nag && is_admin())
			{
				SMLibrary::nag('Upgrade to <strong><a href="http://tinsology.net/plugins/wp-simplemail/" target="_blank">SimpleMail 2</a></strong> for additional features and improvements. <br />
				<a href="http://tinsology.net/plugins/wp-simplemail/" target="_blank">More Info</a>.');
				$this->settings->update_nag = 0;
				update_option('simplemail_settings', $this->settings);
			}
		}
	}
	
	public function connect()
	{
		if(!is_user_logged_in() || !current_user_can('access_email') || !$this->ready())
			return;
			
		$half = false;
		$full = false;

		if((is_admin() || defined('SM_ATTACHMENT')) && SMLibrary::isMailbox())
			$full = true;
		elseif((is_admin() && $this->settings->notifications) || ($this->settings->admin_bar && $this->settings->notifications))
			$half = true;
		elseif(SMLibrary::isSettings())
			$half = true;
		
		if($full)
		{
			$this->current_mailbox = SMLibrary::getMailbox();
			$this->client->connect($this->current_mailbox);
		}
		elseif($half)
			$this->client->quickConnect();
			
		if($this->settings->boxes->inbox)
			$this->client->inbox = $this->settings->boxes->inbox;
	}
	
	public function startClient()
	{
		//prepare flags
		$flags = array();
		foreach($this->settings->flags as $flag => $isset)
		{
			if($isset)
				$flags[] = $flag;
		}
		$this->client = new SimpleMailClient($this->settings->host, $this->settings->username, $this->settings->password, $flags);
	}
	
	public function install()
	{
		//settings
		$defaults = SMLibrary::default_settings();
		
		global $wp_roles;
		$wp_roles->add_cap('administrator', 'access_email');
		
		if(get_option('wpSimpleMailHost'))
		{
			$defaults->legacy = 1;
			
			delete_option('wpSimpleMailDebug');
			delete_option('wpSimpleMailFlags');
			delete_option('wpSimpleMailNotifications');
			delete_option('wpSimpleMailInboxSize');
			delete_option('wpSimpleMailConnSuccess');
			delete_option('wpSimpleMailNormPerm');
			delete_option('wpSimpleMailConfigPerm');
			delete_option('wpSimpleMailVersion');
			delete_option('wpSimpleMailTimeout');
			delete_option('wpSimpleMailDisplayNag');
			delete_option('wpSimpleMailHost');
			delete_option('wpSimpleMailUser');
		}
		
		$defaults->update_nag = 1;
		
		add_option('simplemail_settings', $defaults, null, false);
	}
	
	public function uninstall()
	{
		global $wpdb, $wp_roles;
		
		delete_option('simplemail_settings');
		
		$wp_roles->remove_cap('administrator', 'access_email');
	}
	
	public function ready()
	{
		if($this->settings)
		{
			return $this->settings->ready;
		}
		return false;
	}
}
$sm_plugin = new SimpleMailPlugin();
else :
SMLibrary::nag('<strong>PHP IMAP extension not detected</strong>. WP SimpleMail requires this extension to operate. 
It will have to be installed before you can use this plugin. See the <a href="http://forums.tinsology.net/viewtopic.php?f=16&t=712" target="_blank">Support Forum</a> 
for help installing this extension.');
endif;
?>