<?php
require_once('functions.php');
class SimpleMailMenus
{
	const CAPABILITIES = 'access_email';
	
	private $icon_url;
	
	public function __construct()
	{
		add_action('init', array($this, 'getNag'));
		add_action('init', array($this, 'init'));
		add_action('admin_menu', array($this, 'menus'));
		add_action('admin_print_styles', array($this, 'adminCss'));
	}
	
	public function getNag()
	{
		if(isset($_GET['nag']))
		{
			$messages = array(	'not-ready' 	=> __('SimpleMail requires additional configuration before you can access your email.'),
								'email-deleted' => __('Message Deleted'),
								'message-moved' => __('Message Moved')
									);
										
			if(isset($messages[ $_GET['nag'] ]))
				SMLibrary::nag($messages[ $_GET['nag'] ]);
		}
	}

	public function menus()
	{
		global $sm_plugin;
		$adder = new SimpleMailMenuAdder($this);
		$menus = array();
		
		
		if($sm_plugin->ready())
		{
			$menu_title = __('Email');
			
			if($sm_plugin->client->status->unseen > 0)
				$menu_title .= SMLibrary::format_email_notice($sm_plugin->client->status->unseen);
				
			add_menu_page(__('Email'), $menu_title, self::CAPABILITIES, 'simplemail', array($adder, 'display' . $sm_plugin->client->inbox), $this->icon_url, $sm_plugin->settings->menu_position);
			$menus[] = add_submenu_page('simplemail', __('Inbox'), __('Inbox'), self::CAPABILITIES, 'simplemail', array($adder, 'display' . $sm_plugin->client->inbox));
			
			$compose = add_submenu_page('simplemail', __('Compose'), __('Compose'), self::CAPABILITIES, 'simplemail_compose', array($this, 'displayCompose'));
			add_action('admin_print_styles-' . $compose, array($this, 'composeInit'));

		}
		elseif($sm_plugin->settings->force_login)
		{
			add_menu_page(__('Email'), __('Email'), self::CAPABILITIES, 'simplemail', array($this, 'displayLogin'), $this->icon_url, $sm_plugin->settings->menu_position);
			$login = add_submenu_page('simplemail', __('Login'), __('Login'), self::CAPABILITIES, 'simplemail', array($this, 'displayLogin'));
			add_action('admin_print_styles-' . $login, array($this, 'loginInit'));
		}
		
		$settings = add_options_page(__('Email Settings'), __('Email Settings'), 'manage_options', 'simplemail_settings', array($this, 'displaySettings'));
		add_action('admin_print_styles-' . $settings, array($this, 'settingsInit'));
		
		foreach($menus as $m)
		{
			add_action('admin_print_styles-' . $m, array($this, 'menusInit'));
		}
	}
	
	public function init()
	{
		global $sm_plugin;
		$this->icon_url = $sm_plugin->url . '/ui/images/icon_sm.png';
		$compose = $sm_plugin->url . '/ui/images/compose.png';
		
		if($_SERVER['REQUEST_METHOD'] == 'POST' && isset($_POST['simplemail_settings_save']))
		{
			include('settings_post.php');
		}
		elseif(isset($_POST['compose-submit']))
			include('compose_post.php');
		
		wp_register_script('sm_message_js', $sm_plugin->url . '/ui/js/message.js', array('jquery'), '1.0.1', true);
		wp_register_script('sm_mailbox_js', $sm_plugin->url . '/ui/js/mailbox.js', array('jquery'), '1.0.1', true);
		wp_register_script('sm_lib_js', $sm_plugin->url . '/ui/js/lib.js', array('jquery'), '1.0.1', true);
		wp_register_script('sm_settings_js', $sm_plugin->url . '/ui/js/settings.js', array('jquery'), '1.0.1', true);		
	}
	
	public function adminCss()
	{
		wp_enqueue_style('sm_menu_css', plugins_url('/menu-style.css', __FILE__));
	}
	
	public function settingsInit()
	{
		wp_enqueue_style('sm_css', plugins_url('/style.css', __FILE__));
		wp_enqueue_style('sm_settings_css', plugins_url('/settings.css', __FILE__));
		wp_enqueue_script('sm_settings_js');
	}
	
	public function composeInit()
	{
		wp_enqueue_style('sm_css', plugins_url('/style.css', __FILE__));
	}
	
	public function menusInit()
	{
		wp_enqueue_style('sm_css', plugins_url('/style.css', __FILE__));
		wp_enqueue_script('sm_lib_js');
		if(isset($_GET['uid']))
			wp_enqueue_script('sm_message_js');
		else
			wp_enqueue_script('sm_mailbox_js');
	}
	
	public function loginInit()
	{
		//wp_enqueue_style('sm_login_css', plugins_url('/login.css', __FILE__));
		wp_enqueue_style('sm_css', plugins_url('/style.css', __FILE__));
	}
	
	public function displayLogin()
	{
		include('login.php');
	}
	
	public function displaySettings()
	{
		include('settings.php');
	}
	
	public function displayMailbox($boxName)
	{
		if(isset($_GET['uid']))
			include('message.php');
		else
			include('mailbox.php');
	}
	
	public function displayCompose()
	{
		include('compose.php');
	}
}

new SimpleMailMenus();

class SimpleMailMenuAdder
{
	private $caller;
	
	public function __construct($caller)
	{
		$this->caller = $caller;
	}
	
	public function __call($name, $args)
	{
		$this->caller->displayMailbox($name);
	}
}
?>